//Copyright (c) Microsoft Corporation.  All rights reserved. 
//
// DllSimConnectClient.cpp : Sample of SimConnect client coded as a DLL addon
//
//   To install:
//
//  Edit the dll.xml file for Microsoft ESP (see documentation for location)
//  Add the following section.
//  Make sure the  <Path> item matches the exact location of the dll on your system
//
//<Launch.Addon>
//		<Name>SimConnect DLL Sample</Name>
//		<Disabled>False</Disabled>
//		<ManualLoad>False</ManualLoad>
//		<Path>C:\codeSamples\dllSimConnectClient\DllSimConnectClient.dll</Path>
//</Launch.Addon>


#include "stdafx.h"

#include "simconnect.h"

typedef enum
{
	MAIN_MENU = 1,

	SUB_MENU_1= 100,
	SUB_MENU_2,
	SUB_MENU_3,
	SUB_MENU_4,

	MENU_SELECTION_1 = 200,
	MENU_SELECTION_2,
} EVENT_IDS;

typedef enum
{
	SET_ZULU_HOURS,
	SET_ZULU_MINUTES,
	SET_LOCAL_HOURS,
	SET_LOCAL_MINUTES,
} TRANSMIT_EVENT_IDS;

typedef enum
{
	REQUEST_USER_LLA,
	REQUEST_WEATHER_AT_USER_LLA,
	REQUEST_WEATHER_AT_KSEA,
	REQUEST_WEATHER_AT_NEAREST,
	REQUEST_DISPLAY_WEATHER_OBSERVATION,
} REQUEST_IDS;

typedef enum
{
	DEFINITION_USER_LLA,
} STRUCTURE_IDS;

#pragma pack(push,1)
typedef struct
{
	double Lat;
	double Lon;
	double Alt;
} USER_LLA;
#pragma pack(pop)

HANDLE g_hSimConnect;

void OnRecvOpen (SIMCONNECT_RECV_OPEN *pOpen)
{
	// create menu items
	SimConnect_MenuAddItem(g_hSimConnect, "DllSimConnectClient", MAIN_MENU, 0);
	SimConnect_MenuAddSubItem(g_hSimConnect, MAIN_MENU, "Show Weather Menu", SUB_MENU_1, 0);
	SimConnect_MenuAddSubItem(g_hSimConnect, MAIN_MENU, "Show Time Menu", SUB_MENU_2, 0);

	// register data structures
	SimConnect_AddToDataDefinition(g_hSimConnect, DEFINITION_USER_LLA, "Plane Latitude", "radians");
	SimConnect_AddToDataDefinition(g_hSimConnect, DEFINITION_USER_LLA, "Plane Longitude", "radians");
	SimConnect_AddToDataDefinition(g_hSimConnect, DEFINITION_USER_LLA, "Plane Altitude", "radians");

	// register transmit client events
	SimConnect_MapClientEventToSimEvent(g_hSimConnect, SET_ZULU_HOURS, "ZULU_HOURS_SET");
	SimConnect_MapClientEventToSimEvent(g_hSimConnect, SET_ZULU_MINUTES, "ZULU_MINUTES_SET");
	SimConnect_MapClientEventToSimEvent(g_hSimConnect, SET_LOCAL_HOURS, "CLOCK_HOURS_SET");
	SimConnect_MapClientEventToSimEvent(g_hSimConnect, SET_LOCAL_MINUTES, "CLOCK_MINUTES_SET");
}

char szMenuText1[] = "DllSimConnectClient\0Pick A Weather Option:\0Weather at current location\0Weather at SeaTac\0Close Window\0";
char szMenuText2[] = "DllSimConnectClient\0Pick A Time Option:\0Set Local Time to 12:00\0Set Zulu Time to 00:00\0Close Window\0";

void OnRecvEvent (SIMCONNECT_RECV_EVENT *pEvent)
{
	switch (pEvent->uEventID)
	{
		// process AddOn menu items
		case SUB_MENU_1:
			{
				SimConnect_Text(g_hSimConnect, SIMCONNECT_TEXT_TYPE_MENU, 0.0f, MENU_SELECTION_1, ARRAYSIZE(szMenuText1), szMenuText1);
			}
			break;

		case SUB_MENU_2:
			{
				SimConnect_Text(g_hSimConnect, SIMCONNECT_TEXT_TYPE_MENU, 0.0f, MENU_SELECTION_2, ARRAYSIZE(szMenuText2), szMenuText2);
			}
			break;

		// process item selection from SimConnect_Text for Menu Item 1
		case MENU_SELECTION_1:
			{
				switch (pEvent->dwData)
				{
					case SIMCONNECT_TEXT_RESULT_MENU_SELECT_1:
						{
							SimConnect_RequestDataOnSimObject(g_hSimConnect, REQUEST_USER_LLA, DEFINITION_USER_LLA, SIMCONNECT_OBJECT_ID_USER, SIMCONNECT_PERIOD_ONCE);
						}
						break;

					case SIMCONNECT_TEXT_RESULT_MENU_SELECT_2:
						{
							SimConnect_WeatherRequestObservationAtStation(g_hSimConnect, REQUEST_WEATHER_AT_KSEA, "KSEA");
						}
						break;
				}
			}
			break;

		// process item selection from SimConnect_Text for Menu Item 2
		case MENU_SELECTION_2:
			{
				switch (pEvent->dwData)
				{
					case SIMCONNECT_TEXT_RESULT_MENU_SELECT_1:
						{
							SimConnect_TransmitClientEvent(g_hSimConnect, SIMCONNECT_OBJECT_ID_USER, SET_LOCAL_HOURS, 12, SIMCONNECT_GROUP_PRIORITY_HIGHEST, SIMCONNECT_EVENT_FLAG_GROUPID_IS_PRIORITY);
							SimConnect_TransmitClientEvent(g_hSimConnect, SIMCONNECT_OBJECT_ID_USER, SET_LOCAL_MINUTES, 0, SIMCONNECT_GROUP_PRIORITY_HIGHEST, SIMCONNECT_EVENT_FLAG_GROUPID_IS_PRIORITY);
						}
						break;

					case SIMCONNECT_TEXT_RESULT_MENU_SELECT_2:
						{
							SimConnect_TransmitClientEvent(g_hSimConnect, SIMCONNECT_OBJECT_ID_USER, SET_ZULU_HOURS, 0, SIMCONNECT_GROUP_PRIORITY_HIGHEST, SIMCONNECT_EVENT_FLAG_GROUPID_IS_PRIORITY);
							SimConnect_TransmitClientEvent(g_hSimConnect, SIMCONNECT_OBJECT_ID_USER, SET_ZULU_MINUTES, 0, SIMCONNECT_GROUP_PRIORITY_HIGHEST, SIMCONNECT_EVENT_FLAG_GROUPID_IS_PRIORITY);
						}
						break;
				}
			}
			break;
	}
}

void OnRecvSimobjectData(SIMCONNECT_RECV_SIMOBJECT_DATA *pData)
{
	switch (pData->dwRequestID)
	{
		case REQUEST_USER_LLA:
			{
				USER_LLA *plla = (USER_LLA*)&pData->dwData;
				SimConnect_WeatherRequestInterpolatedObservation(g_hSimConnect, REQUEST_WEATHER_AT_USER_LLA, (float)plla->Lat, (float)plla->Lon, (float)plla->Alt);
			}
			break;
	}
}

void OnRecvWeatherObservation(SIMCONNECT_RECV_WEATHER_OBSERVATION *pWeather)
{
	SimConnect_Text(g_hSimConnect, SIMCONNECT_TEXT_TYPE_SCROLL_YELLOW, 30.0f, REQUEST_DISPLAY_WEATHER_OBSERVATION, strlen(pWeather->szMetar) + 1 , pWeather->szMetar);
}

// This callback processes all SimConnect related messages
// It is similar to the call in the dispatch handler (message pump) in a windows application
// But the interesting part is that no message pump is required for the dll
void CALLBACK SimConnectProcess(SIMCONNECT_RECV *pData, DWORD cbData, void *pContext)
{
	// process SIMCONNECT_RECV_ID_XXX values here as needed
	switch(pData->dwID)
	{
		case SIMCONNECT_RECV_ID_OPEN:
			OnRecvOpen((SIMCONNECT_RECV_OPEN*)pData);
			break;

		case SIMCONNECT_RECV_ID_EVENT:
			OnRecvEvent((SIMCONNECT_RECV_EVENT*)pData);
			break;

		case SIMCONNECT_RECV_ID_SIMOBJECT_DATA:
			OnRecvSimobjectData((SIMCONNECT_RECV_SIMOBJECT_DATA*)pData);
			break;

		case SIMCONNECT_RECV_ID_WEATHER_OBSERVATION:
			OnRecvWeatherObservation((SIMCONNECT_RECV_WEATHER_OBSERVATION*)pData);
			break;
	}
}

void DLLStart(void)
{
	// open connection to local SimConnect server
	SimConnect_Open(&g_hSimConnect, "DllSimConnectClient", NULL, 0, NULL, SIMCONNECT_OPEN_CONFIGINDEX_LOCAL);

	// register callback routine for message processing
	SimConnect_CallDispatch(g_hSimConnect, SimConnectProcess, NULL);
}

void DLLStop(void)
{
	// close connection to local SimConnect server
	SimConnect_Close(g_hSimConnect);
}